from typing import Generator

from fastapi import Depends, HTTPException, status, Body, Request
from fastapi.security import OAuth2PasswordBearer
from jose import jwt
from pydantic import ValidationError
from sqlalchemy.orm import Session

from jwtserver import crud, models, schemas
from jwtserver.core import security
from jwtserver.core.config import settings
from jwtserver.db.session import SessionLocal
from jwtserver.recaptcha.google import RecaptchaGoogleV3

reusable_oauth2 = OAuth2PasswordBearer(
    tokenUrl=f"{settings.API_V1_STR}/login/access-token"
)


def get_db() -> Generator:
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


def get_current_user(
    db: Session = Depends(get_db), token: str = Depends(reusable_oauth2)
) -> models.User:
    try:
        payload = jwt.decode(
            token, settings.SECRET_KEY, algorithms=[security.ALGORITHM]
        )
        token_data = schemas.TokenPayload(**payload)
    except (jwt.JWTError, ValidationError):
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Could not validate credentials",
        )
    user = crud.user.get(db, id=token_data.sub)
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    return user


def get_current_active_user(
    current_user: models.User = Depends(get_current_user),
) -> models.User:
    if not crud.user.is_active(current_user):
        raise HTTPException(status_code=400, detail="Inactive user")
    return current_user


def get_current_active_superuser(
    current_user: models.User = Depends(get_current_user),
) -> models.User:
    if not crud.user.is_superuser(current_user):
        raise HTTPException(
            status_code=400, detail="The user doesn't have enough privileges"
        )
    return current_user


async def recaptcha_google_v3_action(request: Request, recaptcha=Body(...)):
    # if self.environment == "tests":
    #     self.r_json = {
    #         "success": self.recaptcha_token.split(":")[0],
    #         "action": self.recaptcha_token.split(":")[1],
    #         "score": float(self.recaptcha_token.split(":")[2]),
    #     }
    #     return self
    recaptcha = RecaptchaGoogleV3(
        _config=settings.google.recaptcha,
        recaptcha_token=recaptcha,
    )
    await recaptcha.set_action_name(request.url.path).greenlight()

    request.state.recaptcha = recaptcha
